from geopy.geocoders import Nominatim
from types import SimpleNamespace
from unidecode import unidecode
import json

states_abbreviation = {'ACRE':'AC',
'ALAGOAS':'AL',
'AMAZONAS':'AM',
'AMAPA':'AP',
'BAHIA':'BA',
'CEARA':'CE',
'DISTRITO FEDERAL':'DF',
'ESPÍRITO SANTO':'ES',
'GOIAS':'GO',
'MARANHAO':'MA',
'MINAS GERAIS':'MG',
'MATO GROSSO DO SUL':'MS',
'MATO GROSSO':'MT',
'PARA':'PA',
'PARAIBA':'PB',
'PERNAMBUCO':'PE',
'PIAUI':'PI',
'PARANA':'PR',
'RIO DE JANEIRO':'RJ',
'RIO GRANDE DO NORTE':'RN',
'RONDONIA':'RO',
'RORAIMA':'RR',
'RIO GRANDE DO SUL':'RS',
'SANTA CATARINA':'SC',
'SERGIPE':'SE',
'SAO PAULO':'SP',
'TOCANTIS':'TO'}

def get_state_abbreviation(state_name):
    return states_abbreviation[unidecode(state_name).upper()]
    
def get_address(user_agent,coords):
    geolocator = Nominatim(user_agent=user_agent)
    return json_to_object(json.dumps(geolocator.reverse(coords).raw))

def json_to_object(data):
    return json.loads(data, object_hook=lambda d: SimpleNamespace(**d))

async def async_http_request(session, url, params=None):
    async with session.get(url, params=params) as res:
        res.raise_for_status()
        current = await res.text()
    return current



API_URL = "http://apiadvisor.climatempo.com.br"
TOKEN_PARAMETER_STR = "token"
LATITUDE_PARAMETER_STR = "latitude"
LONGITUDE_PARAMETER_STR = "longitude"
CURRENT_WEATHER_URL="/api/v1/weather/locale/{}/current"
FORECAST_72_HOURS_URL = "/api/v1/forecast/locale/{}/hours/72"
FORECAST_15_DAYS_URL = "/api/v1/forecast/locale/{}/days/15"
SEARCH_LOCALE_URL = "/api/v1/locale/city"



from .const import *
from  climatempopy import common

class ClimaTempo:    

    def __init__(self, session, climatempo_token, locale_id=None):
        self.session = session
        self.climatempo_token = climatempo_token
        self.locale_id = locale_id
        self.current = None
        self.forecast_72_hours= None
        self.forecast_15_days= None

    async def get_current_weather(self):
        url = API_URL+CURRENT_WEATHER_URL.format(self.locale_id)
        return await self.get_climatempo_object_data(url)

    async def get_72_hours_forecast(self):
        url = API_URL+FORECAST_72_HOURS_URL.format(self.locale_id)
        return await self.get_climatempo_object_data(url)

    async def get_15_days_forecast(self):
        url = API_URL+FORECAST_15_DAYS_URL.format(self.locale_id)
        return await self.get_climatempo_object_data(url)

    async def get_locales(self,city,state=None,country=None):
        url = API_URL+SEARCH_LOCALE_URL.format(self.locale_id)
        params = {}
        params["name"]=city.upper()
        if state:
            params["state"]=state.upper()
        if country:
            params["country"]=country.upper()
        return await self.get_climatempo_object_data(url,params)

    async def get_climatempo_object_data(self, url, params=None):
        if not params:
            params = {}
        params[TOKEN_PARAMETER_STR]=self.climatempo_token
        current = await common.async_http_request(self.session,url,params)
        return common.json_to_object(current) 
    
    async def update_current_weather(self):
        self.current = await self.get_current_weather()

    async def update_72_hours_forecast (self):
        self.forecast_72_hours = await self.get_72_hours_forecast()
    
    async def update_15_days_forecast (self):
        self.forecast_15_days = await self.get_15_days_forecast()
    
    async def set_locale_id_from_coords(self,latitude,longitude):
        locale = await self.get_locale_from_coords(latitude,longitude)
        self.locale_id= locale[0].id

    async def get_locale_from_coords(self,latitude,longitude):
        location=common.get_address("climatempopy_lib",coords='{},{}'.format(latitude,longitude))
        return await self.get_locales(location.address.city,common.get_state_abbreviation(location.address.state),location.address.country_code)